@@ -1,6 +1,6 @@
 /* 
    HTTP session handling
-   Copyright (C) 1999-2009, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2021, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -33,13 +33,17 @@ NE_BEGIN_DECLS
 
 typedef struct ne_session_s ne_session;
 
-/* Create a session to the given server, using the given scheme.  If
- * "https" is passed as the scheme, SSL will be used to connect to the
- * server. */
-ne_session *ne_session_create(const char *scheme,
-			      const char *hostname, unsigned int port);
+/* Create a session to the server 'host', using the given scheme.  If
+ * "https" is passed as the scheme, TLS will be used to connect to the
+ * server. The host string must follow the definition of 'host' in RFC
+ * 3986, which can be an IP-literal or registered (DNS) hostname. An
+ * IPv6 literal address must be enclosed in square brackets (for
+ * example "[::1]"). The RFC 6874 syntax for IPv6 link-local literal
+ * addresses is also supported, for example "[fe80::1%25eth0]". */
+ne_session *ne_session_create(const char *scheme, const char *host,
+                              unsigned int port);
 
-/* Finish an HTTP session */
+/* Finish an HTTP session, freeing associated memory. */
 void ne_session_destroy(ne_session *sess);
 
 /* Prematurely force the connection to be closed for the given
@@ -82,8 +86,7 @@ typedef enum ne_session_flag_e {
     NE_SESSFLAG_ICYPROTO, /* enable this flag to enable support for
                            * non-HTTP ShoutCast-style "ICY" responses. */
 
-    NE_SESSFLAG_SSLv2, /* disable this flag to disable support for
-                        * SSLv2, if supported by the SSL library. */
+    NE_SESSFLAG_SSLv2, /* this flag is ignored */
 
     NE_SESSFLAG_RFC4918, /* enable this flag to enable support for
                           * RFC4918-only WebDAV features; losing
@@ -104,6 +107,10 @@ typedef enum ne_session_flag_e {
                              * to improve interoperability with
                              * SharePoint */
 
+    NE_SESSFLAG_STRICT, /* disable this flag to parse HTTP/1.1
+                         * messages without strict requirements
+                         * introduced in RFC 7230 and later. */
+
     NE_SESSFLAG_LAST /* enum sentinel value */
 } ne_session_flag;
 
@@ -281,9 +288,19 @@ void ne_ssl_set_clicert(ne_session *sess, const ne_ssl_client_cert *clicert);
 void ne_ssl_trust_cert(ne_session *sess, const ne_ssl_certificate *cert);
 
 /* If the SSL library provided a default set of CA certificates, trust
- * this set of CAs. */
+ * this set of CAs. This function has no effect for non-SSL
+ * sessions. */
 void ne_ssl_trust_default_ca(ne_session *sess);
 
+/* Set the minimum and maximum SSL/TLS protocol version for the
+ * session. Either minimum and/or maximum may be specified as
+ * NE_SSL_PROTO_UNSPEC, in which case no limit is imposed. Returns
+ * non-zero on error if the range is invalid, if the range cannot be
+ * configured in the SSL/TLS toolkit, or if SSL/TLS is not in use for
+ * the session. */
+int ne_ssl_set_protovers(ne_session *sess, enum ne_ssl_protocol min,
+                         enum ne_ssl_protocol max);
+
 /* Callback used to load a client certificate on demand.  If dncount
  * is > 0, the 'dnames' array dnames[0] through dnames[dncount-1]
  * gives the list of CA names which the server indicated were
@@ -309,10 +326,10 @@ void ne_set_connect_timeout(ne_session *sess, int timeout);
 /* Sets the user-agent string. neon/VERSION will be appended, to make
  * the full header "User-Agent: product neon/VERSION".
  * If this function is not called, the User-Agent header is not sent.
- * The product string must follow the RFC2616 format, i.e.
+ * The product string must follow the RFC 7231§5.5.3 format, i.e.
  *       product         = token ["/" product-version]
  *       product-version = token
- * where token is any alpha-numeric-y string [a-zA-Z0-9]* */
+ * where token is an alphanumeric string. */
 void ne_set_useragent(ne_session *sess, const char *product);
 
 /* Returns non-zero if next-hop server does not claim compliance to
