@@ -198,6 +198,9 @@ int ne_uri_parse(const char *uri, ne_uri *parsed)
         if (s[0] == '[') {
             p = s + 1;
 
+            /* This allows any characters in IP-literal which is too
+             * broad, however e.g. zone identifiers per RFC 6874 are
+             * allowed through. */
             while (*p != ']' && p < pa)
                 p++;
 
@@ -206,14 +209,14 @@ int ne_uri_parse(const char *uri, ne_uri *parsed)
                 return -1;
             }
 
-            p++; /* => p = colon */
+            p++; /* => p = [ ':' port ] */
         } else {
-            /* Find the colon. */
+            /* Find any colon before reaching path-abempty. */
             p = s;
             while (*p != ':' && p < pa)
                 p++;
         }
-        /* => p = colon */
+        /* => p = [ ":" port ] */
 
         parsed->host = ne_strndup(s, p - s);
 
